#
# $Id: drmaa.core.pxd 533 2007-12-22 15:25:42Z lukasz $
#
# Python bindings to DRMAA
# Copyright (C) 2006-2007  Fedstage Systems Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

cdef extern from *:
	ctypedef int size_t

cdef extern from "stdio.h":
	int printf( char *fmt, ... )

cdef extern from "stdlib.h":
	void *malloc( size_t size )
	void *calloc( size_t nmemb, size_t size )
	void free( void *ptr )
	void *realloc( void *ptr, size_t size )

cdef extern from "drmaa.h":
	ctypedef struct drmaa_job_template_t
	ctypedef struct drmaa_attr_names_t
	ctypedef struct drmaa_attr_values_t
	ctypedef struct drmaa_job_ids_t
	ctypedef enum:
		DRMAA_ATTR_BUFFER
		DRMAA_CONTACT_BUFFER
		DRMAA_DRM_SYSTEM_BUFFER
		DRMAA_DRMAA_IMPL_BUFFER
		DRMAA_ERROR_STRING_BUFFER
		DRMAA_JOBNAME_BUFFER
		DRMAA_SIGNAL_BUFFER

	cdef enum:
		DRMAA_TIMEOUT_NO_WAIT          #= 0
		DRMAA_TIMEOUT_WAIT_FOREVER     #= -1
	cdef enum:
		DRMAA_PS_UNDETERMINED          #= 0x00
		DRMAA_PS_QUEUED_ACTIVE         #= 0x10
		DRMAA_PS_SYSTEM_ON_HOLD        #= 0x11
		DRMAA_PS_USER_ON_HOLD          #= 0x12
		DRMAA_PS_USER_SYSTEM_ON_HOLD   #= 0x13
		DRMAA_PS_RUNNING               #= 0x20
		DRMAA_PS_SYSTEM_SUSPENDED      #= 0x21
		DRMAA_PS_USER_SUSPENDED        #= 0x22
		DRMAA_PS_USER_SYSTEM_SUSPENDED #= 0x23
		DRMAA_PS_DONE                  #= 0x30
		DRMAA_PS_FAILED                #= 0x40
	cdef enum:
		DRMAA_CONTROL_SUSPEND          #= 0
		DRMAA_CONTROL_RESUME           #= 1
		DRMAA_CONTROL_HOLD             #= 2
		DRMAA_CONTROL_RELEASE          #= 3
		DRMAA_CONTROL_TERMINATE        #= 4
	cdef enum:
		DRMAA_ERRNO_SUCCESS                            #=  0
		DRMAA_ERRNO_INTERNAL_ERROR                     #=  1
		DRMAA_ERRNO_DRM_COMMUNICATION_FAILURE          #=  2
		DRMAA_ERRNO_AUTH_FAILURE                       #=  3
		DRMAA_ERRNO_INVALID_ARGUMENT                   #=  4
		DRMAA_ERRNO_NO_ACTIVE_SESSION                  #=  5
		DRMAA_ERRNO_NO_MEMORY                          #=  6
		DRMAA_ERRNO_INVALID_CONTACT_STRING             #=  7
		DRMAA_ERRNO_DEFAULT_CONTACT_STRING_ERROR       #=  8
		DRMAA_ERRNO_NO_DEFAULT_CONTACT_STRING_SELECTED #=  9
		DRMAA_ERRNO_DRMS_INIT_FAILED                   #= 10
		DRMAA_ERRNO_ALREADY_ACTIVE_SESSION             #= 11
		DRMAA_ERRNO_DRMS_EXIT_ERROR                    #= 12
		DRMAA_ERRNO_INVALID_ATTRIBUTE_FORMAT           #= 13
		DRMAA_ERRNO_INVALID_ATTRIBUTE_VALUE            #= 14
		DRMAA_ERRNO_CONFLICTING_ATTRIBUTE_VALUES       #= 15
		DRMAA_ERRNO_TRY_LATER                          #= 16
		DRMAA_ERRNO_DENIED_BY_DRM                      #= 17
		DRMAA_ERRNO_INVALID_JOB                        #= 18
		DRMAA_ERRNO_RESUME_INCONSISTENT_STATE          #= 19
		DRMAA_ERRNO_SUSPEND_INCONSISTENT_STATE         #= 20
		DRMAA_ERRNO_HOLD_INCONSISTENT_STATE            #= 21
		DRMAA_ERRNO_RELEASE_INCONSISTENT_STATE         #= 22
		DRMAA_ERRNO_EXIT_TIMEOUT                       #= 23
		DRMAA_ERRNO_NO_RUSAGE                          #= 24
		DRMAA_ERRNO_NO_MORE_ELEMENTS                   #= 25

	int drmaa_init(
			char *contact,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_exit( char *error_diagnosis, size_t error_diag_len )

	int drmaa_allocate_job_template(
			drmaa_job_template_t **jt,
			char *error_diagnosis,
			size_t error_diag_len
			)

	int drmaa_delete_job_template(
			drmaa_job_template_t *jt,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_set_attribute(
			drmaa_job_template_t *jt,
			char *name, char *value,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_get_attribute(
			drmaa_job_template_t *jt,
			char *name, char *value, size_t value_len,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_set_vector_attribute(
			drmaa_job_template_t *jt,
			char *name, char *value[],
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_get_vector_attribute(
			drmaa_job_template_t *jt,
			char *name, drmaa_attr_values_t **values,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_get_attribute_names(
			drmaa_attr_names_t **values,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_get_vector_attribute_names(
			drmaa_attr_names_t **values,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_get_next_attr_name(
			drmaa_attr_names_t* values,
			char *value, size_t value_len
			)

	int drmaa_get_next_attr_value(
			drmaa_attr_values_t* values,
			char *value, size_t value_len
			)

	int drmaa_get_next_job_id(
			drmaa_job_ids_t* values,
			char *value, size_t value_len
			)

	int drmaa_get_num_attr_names( drmaa_attr_names_t* values, size_t *size )

	int drmaa_get_num_attr_values( drmaa_attr_values_t* values, size_t *size )

	int drmaa_get_num_job_ids( drmaa_job_ids_t* values, size_t *size )

	void drmaa_release_attr_names( drmaa_attr_names_t* values )

	void drmaa_release_attr_values( drmaa_attr_values_t* values )

	void drmaa_release_job_ids( drmaa_job_ids_t* values )

	int drmaa_run_job(
			char *job_id, size_t job_id_len, drmaa_job_template_t *jt,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_run_bulk_jobs(
			drmaa_job_ids_t **jobids,
			drmaa_job_template_t *jt,
			int start, int end, int incr,
			char *error_diagnosis,
			size_t error_diag_len
			)

	int drmaa_control(
			char *job_id, int action,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_job_ps(
			char *job_id, int *remote_ps,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_synchronize(
			char *job_ids[], signed long timeout, int dispose,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_wait(
			char *job_id,
			char *job_id_out, size_t job_id_out_len, int *stat,
			signed long timeout, drmaa_attr_values_t **rusage,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_wifexited(
			int *exited, int stat,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_wexitstatus(
			int *exit_status, int stat,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_wifsignaled(
			int *signaled, int stat,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_wtermsig(
			char *signal, size_t signal_len, int stat,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_wcoredump(
			int *core_dumped, int stat,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_wifaborted(
			int *aborted, int stat,
			char *error_diagnosis, size_t error_diag_len
			)

	char *drmaa_strerror( int drmaa_errno )

	int drmaa_get_contact(
			char *contact, size_t contact_len,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_version(
			unsigned int *major, unsigned int *minor,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_get_DRM_system(
			char *drm_system, size_t drm_system_len,
			char *error_diagnosis, size_t error_diag_len
			)

	int drmaa_get_DRMAA_implementation(
			char *drmaa_impl, size_t drmaa_impl_len,
			char *error_diagnosis, size_t error_diag_len
			)


cdef extern from "Python.h":
	ctypedef class __builtin__.Exception [object PyBaseExceptionObject]:
		pass

cdef class drmaa_error( Exception ):
	cdef int _code
	cdef object _message

